package hr.geometrics;

import java.io.Serializable;

public class Direction implements Serializable {
	
	private static final long serialVersionUID = 7787781361406547228L;

	/**
	 * Parameter A (Ax + By + C = 0)
	 */
	private double A;
	
	/**
	 * Parameter B (Ax + By + C = 0)
	 */
	private double B;
	
	/**
	 * Parameter C (Ax + By + C = 0)
	 */
	private double C;
	
	/**
	 * Creates direction from line
	 * @param origin Line that lies on wanted Direction
	 */
	public Direction(Line origin) {
		
		this.A = origin.getY1()-origin.getY2();
		this.B = origin.getX2()-origin.getX1();
		this.C = origin.getX1()*(-1)*this.getA() + origin.getY1()*(-1)*this.getB();
		
	}

	/**
	 * Creates direction from point and angle (in radians)
	 * @param point Point that lies on direction
	 * @param angle angle that direction closes with x-axis (smaller one, from -pi/2 to pi/2)
	 */
	public Direction(Point point, double angle) {
		
		// for -90|90 degrees
		if (Math.abs(angle - Math.PI/2) < 10E-6 || Math.abs(angle + Math.PI/2) < 10E-6) {
			this.A = 1;
			this.B = 0;
			this.C = point.getX();
		} else {
			this.A = -Math.tan(angle);
			this.B = 1;
			this.C = Math.tan(angle)*point.getX() - point.getY();
		}
		
	}
	
	public Direction(double A, double B, double C) {
		this.A = A;
		this.B = B;
		this.C =  C;
	}
	
	/**
	 * Returns angle tangens (from -infinity to infinity)
	 * @return
	 */
	public double getAngleTangens() {
		if (this.B == 0)
			return Double.POSITIVE_INFINITY;
		
		return -this.A/this.B;
	}
	
	public Point getIntersectionWith(Line line) {
		
		Point possibleIntersection = this.getIntersectionWith(line.getDirection());
		
		if (possibleIntersection == null) {
			return null;
		}
		
		if (possibleIntersection.getX() >= Math.min(line.getX1(), line.getX2())
			&& possibleIntersection.getX() <= Math.max(line.getX1(), line.getX2())
			&& possibleIntersection.getY() >= Math.min(line.getY1(), line.getY2())
			&& possibleIntersection.getY() <= Math.max(line.getY1(), line.getY2())
		) {
			return possibleIntersection;
		} 
		
		return null;
		
	}
	
	public Point getIntersectionWith(Direction direction) {
		
		Point intersection = null;
		
		double[][] matrix = new double[][] {
							{ this.getA(), this.getB(), -this.getC()},
							{ direction.getA(), direction.getB(), -direction.getC()}
		};
		
		// Cramer's rule
		double denominator = matrix[0][0]*matrix[1][1] - matrix[0][1]*matrix[1][0];
		if (denominator == 0) {
			return null;
		}
		double x = (matrix[0][2]*matrix[1][1] - matrix[0][1]*matrix[1][2]) / denominator; 
		double y = (matrix[0][0]*matrix[1][2] - matrix[0][2]*matrix[1][0]) / denominator;
		
		intersection = new Point(x, y);		
		
		//System.out.println(this+" and "+direction+ " -> "+intersection);
		
		return intersection;
		
	}
	
	/**
	 * Returns angle from -pi/2 to pi/2
	 * @return
	 */
	public double getAngle() {
		return Math.atan(this.getAngleTangens());	
	}

	/**
	 * @return the a
	 */
	public double getA() {
		return A;
	}

	/**
	 * @return the b
	 */
	public double getB() {
		return B;
	}

	/**
	 * @return the c
	 */
	public double getC() {
		return C;
	}
	
	public String toString() {
		
		return "(" + this.getA() + ")x + (" + this.getB() + ")y + (" + this.getC() + ") = 0";
		
	}
	
}
